window.Gleam.initSearch([{"doc":"glimit","title":"glimit","content":"# glimit\n\n[![Package Version](https://img.shields.io/hexpm/v/glimit)](https://hex.pm/packages/glimit)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glimit/)\n\nA framework-agnostic rate limiter for Gleam. 💫\n\n&gt; ⚠️  This library is still in development, use at your own risk.\n\n\n## Installation\n\n```sh\ngleam add glimit\n```\n\n\n## Example usage\n\nFor example, `glimit` could be used to rate limit requests to a mist HTTP server:\n\n```gleam\nimport glimit\n\n\nfn handle_request(req: Request(Connection)) -&gt; Response(ResponseData) {\n  let index =\n    response.new(200)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n  let not_found =\n    response.new(404)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n\n  case request.path_segments(req) {\n    [] -&gt; index\n    _ -&gt; not_found\n  }\n}\n\nfn get_identifier(req: Request(Connection)) -&gt; Result(String, String) {\n  req.body\n  |&gt; get_client_info\n  |&gt; result.map(fn(client_info: ConnectionInfo) {\n    client_info.ip_address |&gt; string.inspect\n  })\n  |&gt; result.unwrap(&quot;unknown IP address&quot;)\n}\n\npub fn main() {\n  let rate_limit_reached = fn(_req) -&gt; {\n    response.new(429)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n  }\n\n  let limiter =\n    glimit.new()\n    |&gt; glimit.per_second(10)\n    |&gt; glimit.per_minute(100)\n    |&gt; glimit.per_hour(1000)\n    |&gt; glimit.identifier(get_identifier)\n    |&gt; glimit.handler(rate_limit_reached)\n    |&gt; glimit.build\n\n  let assert Ok(_) =\n    handle_request\n    |&gt; glimit.apply(limiter)\n    |&gt; mist.new\n    |&gt; mist.port(8080)\n    |&gt; mist.start_http\n\n  process.sleep_forever()\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/glimit&gt;.\n\n## Development\n\n```sh\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"glimit/utils","title":"now","content":"pub fn now() -&gt; Int\n Get the current time in epoch seconds.\nSynonyms:\nglimit/utils.now\nglimit/utils now","url":"glimit/utils.html#now"},{"doc":"glimit/utils","title":"glimit/utils","content":" A module containing utility functions.\n","url":"glimit/utils.html"},{"doc":"glimit","title":"Message","content":"pub type Message(a) {\n  Shutdown\n  Hit(input: a, reply_with: Subject(Result(Nil, Nil)))\n}\n The messages that the actor can receive.\n\nShutdown\n Stop the actor.\n\nHit(input: a, reply_with: Subject(Result(Nil, Nil)))\n Mark a hit for a given identifier.\n\nSynonyms:\nglimit.Message\nglimit Message","url":"glimit.html#Message"},{"doc":"glimit","title":"RateLimiter","content":"pub type RateLimiter(a, b) {\n  RateLimiter(subject: Subject(Message(a)), handler: fn(a) -&gt; b)\n}\n The rate limiter&#39;s public interface.\n\nRateLimiter(subject: Subject(Message(a)), handler: fn(a) -&gt; b)\n\n\nSynonyms:\nglimit.RateLimiter\nglimit RateLimiter","url":"glimit.html#RateLimiter"},{"doc":"glimit","title":"RateLimiterBuilder","content":"pub type RateLimiterBuilder(a, b, id) {\n  RateLimiterBuilder(\n    per_second: Option(Int),\n    per_minute: Option(Int),\n    per_hour: Option(Int),\n    identifier: fn(a) -&gt; id,\n    handler: fn(a) -&gt; b,\n  )\n}\n A rate limiter.\n\nRateLimiterBuilder(\n  per_second: Option(Int),\n  per_minute: Option(Int),\n  per_hour: Option(Int),\n  identifier: fn(a) -&gt; id,\n  handler: fn(a) -&gt; b,\n)\n\n\nSynonyms:\nglimit.RateLimiterBuilder\nglimit RateLimiterBuilder","url":"glimit.html#RateLimiterBuilder"},{"doc":"glimit","title":"State","content":"pub type State(a, b, id) {\n  RateLimiterState(\n    hit_log: dict.Dict(id, List(Int)),\n    per_second: Option(Int),\n    per_minute: Option(Int),\n    per_hour: Option(Int),\n    identifier: fn(a) -&gt; id,\n    handler: fn(a) -&gt; b,\n  )\n}\n The actor state of the actor.\n\n The state is a dictionary where the key is the identifier and the value is a list of epoch timestamps.\n\nRateLimiterState(\n  hit_log: dict.Dict(id, List(Int)),\n  per_second: Option(Int),\n  per_minute: Option(Int),\n  per_hour: Option(Int),\n  identifier: fn(a) -&gt; id,\n  handler: fn(a) -&gt; b,\n)\n\n\nSynonyms:\nglimit.State\nglimit State","url":"glimit.html#State"},{"doc":"glimit","title":"apply","content":"pub fn apply(\n  func: fn(a) -&gt; b,\n  limiter: RateLimiter(a, b),\n) -&gt; fn(a) -&gt; b\n Apply the rate limiter to a request handler or function.\n\nSynonyms:\nglimit.apply\nglimit apply","url":"glimit.html#apply"},{"doc":"glimit","title":"build","content":"pub fn build(\n  config: RateLimiterBuilder(a, b, c),\n) -&gt; RateLimiter(a, b)\n Build the rate limiter.\n\nSynonyms:\nglimit.build\nglimit build","url":"glimit.html#build"},{"doc":"glimit","title":"handler","content":"pub fn handler(\n  limiter: RateLimiterBuilder(a, b, c),\n  handler: fn(a) -&gt; b,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the handler to be called when the rate limit is reached.\n\nSynonyms:\nglimit.handler\nglimit handler","url":"glimit.html#handler"},{"doc":"glimit","title":"identifier","content":"pub fn identifier(\n  limiter: RateLimiterBuilder(a, b, c),\n  identifier: fn(a) -&gt; c,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the identifier function to be used to identify the rate limit.\n\nSynonyms:\nglimit.identifier\nglimit identifier","url":"glimit.html#identifier"},{"doc":"glimit","title":"new","content":"pub fn new() -&gt; RateLimiterBuilder(a, b, c)\n Create a new rate limiter builder.\n\n Panics when the rate limit hit counter cannot be created.\n\nSynonyms:\nglimit.new\nglimit new","url":"glimit.html#new"},{"doc":"glimit","title":"per_hour","content":"pub fn per_hour(\n  limiter: RateLimiterBuilder(a, b, c),\n  limit: Int,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the rate limit per hour.\n\nSynonyms:\nglimit.per_hour\nglimit per_hour","url":"glimit.html#per_hour"},{"doc":"glimit","title":"per_minute","content":"pub fn per_minute(\n  limiter: RateLimiterBuilder(a, b, c),\n  limit: Int,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the rate limit per minute.\n\nSynonyms:\nglimit.per_minute\nglimit per_minute","url":"glimit.html#per_minute"},{"doc":"glimit","title":"per_second","content":"pub fn per_second(\n  limiter: RateLimiterBuilder(a, b, c),\n  limit: Int,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the rate limit per second.\n\nSynonyms:\nglimit.per_second\nglimit per_second","url":"glimit.html#per_second"},{"doc":"glimit","title":"stop","content":"pub fn stop(limiter: RateLimiter(a, b)) -&gt; Nil\n Stop the rate limiter agent.\n\nSynonyms:\nglimit.stop\nglimit stop","url":"glimit.html#stop"},{"doc":"glimit","title":"glimit","content":" A framework-agnostic rate limiter.\n","url":"glimit.html"}]);